"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var PaymentsController_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.PaymentsController = void 0;
const common_1 = require("@nestjs/common");
const jwt = require("jsonwebtoken");
const mongoose_1 = require("@nestjs/mongoose");
const mongoose_2 = require("mongoose");
const payment_schema_1 = require("../script/schema/payment.schema");
let PaymentsController = PaymentsController_1 = class PaymentsController {
    constructor(paymentModel) {
        this.paymentModel = paymentModel;
        this.logger = new common_1.Logger(PaymentsController_1.name);
        this.logger.log('Payment Controller initialized');
    }
    async handleWebhook(body, res) {
        this.logger.log('Received webhook notification from App Store');
        const notification = body;
        console.log('notification : ', body);
        try {
            const signedPayload = notification.signedPayload;
            this.logger.debug(`Decoding signed payload: ${signedPayload}`);
            const decodedPayload = jwt.decode(signedPayload);
            const { notificationType, subtype } = decodedPayload;
            console.log('Decoded Payload : ', decodedPayload);
            this.logger.log(`Notification Type: ${notificationType}, Subtype: ${subtype}`);
            switch (notificationType) {
                case 'SUBSCRIBED':
                    if (subtype === 'INITIAL_BUY') {
                        await this.handleInitialPurchase(decodedPayload);
                    }
                    else if (subtype === 'RESUBSCRIBE') {
                        await this.handleResubscribe(decodedPayload);
                    }
                    break;
                case 'DID_RENEW':
                    await this.handleDidRenew(decodedPayload);
                    break;
                case 'EXPIRED':
                    await this.handleVoluntaryExpire(decodedPayload);
                    break;
                case 'DID_FAIL_TO_RENEW':
                    if (subtype === 'GRACE_PERIOD') {
                        await this.handleGracePeriod(decodedPayload);
                    }
                    else {
                        await this.handleFailedRenewal(decodedPayload);
                    }
                    break;
                case 'DID_CHANGE_RENEWAL_STATUS':
                    if (subtype === 'AUTO_RENEW_DISABLED') {
                        await this.handleAutoRenewDisabled(decodedPayload);
                    }
                    break;
                default:
                    this.logger.warn(`Unhandled notification type: ${notificationType}`);
            }
            res.status(200).send('Notification received and processed successfully');
        }
        catch (error) {
            this.logger.error('Error processing notification', error.stack);
            res.status(500).send('Error processing notification');
        }
    }
    async handleResubscribe(decodedPayload) {
        console.log('handleResubscribe');
        this.logger.log('Handling resubscribe event');
        const transactionInfo = jwt.decode(decodedPayload.data.signedTransactionInfo);
        this.logger.debug(`Resubscribe transaction info: ${JSON.stringify(transactionInfo)}`);
        await this.paymentModel.findOneAndUpdate({ purchaseId: transactionInfo.originalTransactionId }, {
            $set: {
                status: true,
                ExpireDate: new Date(transactionInfo.expiresDate),
                isExpire: false,
            },
        }, { new: true });
        this.logger.log('Database updated for resubscribe');
    }
    async handleInitialPurchase(decodedPayload) {
        this.logger.log('Handling initial purchase event');
        const transactionInfo = jwt.decode(decodedPayload.data.signedTransactionInfo);
        this.logger.debug(`Initial purchase transaction info: ${JSON.stringify(transactionInfo)}`);
        this.logger.warn(`Initial purchase already handled via '/storePayment'. Skipping database operation.`);
    }
    async handleDidRenew(decodedPayload) {
        this.logger.log('Handling subscription renewal event');
        const transactionInfo = jwt.decode(decodedPayload.data.signedTransactionInfo);
        this.logger.debug(`Renewal transaction info: ${JSON.stringify(transactionInfo)}`);
        await this.paymentModel.findOneAndUpdate({ purchaseId: transactionInfo.originalTransactionId }, {
            $set: {
                ExpireDate: new Date(transactionInfo.expiresDate),
                status: true,
                isExpire: false,
            },
        }, { new: true });
        this.logger.log('Database updated for subscription renewal');
    }
    async handleVoluntaryExpire(decodedPayload) {
        this.logger.log('Handling voluntary expiration event');
        const transactionInfo = jwt.decode(decodedPayload.data.signedTransactionInfo);
        this.logger.debug(`Voluntary expiration transaction info: ${JSON.stringify(transactionInfo)}`);
        await this.paymentModel.findOneAndUpdate({ purchaseId: transactionInfo.originalTransactionId }, {
            $set: {
                status: false,
                isExpire: true,
            },
        }, { new: true });
        this.logger.log('Database updated for voluntary expiration');
    }
    async handleFailedRenewal(decodedPayload) {
        this.logger.log('Handling failed renewal event');
        const transactionInfo = jwt.decode(decodedPayload.data.signedTransactionInfo);
        this.logger.debug(`Failed renewal transaction info: ${JSON.stringify(transactionInfo)}`);
        await this.paymentModel.findOneAndUpdate({ purchaseId: transactionInfo.originalTransactionId }, {
            $set: {
                status: false,
            },
        }, { new: true });
        this.logger.log('Database updated for failed renewal');
    }
    async handleGracePeriod(decodedPayload) {
        this.logger.log('Handling grace period event');
        const transactionInfo = jwt.decode(decodedPayload.data.signedTransactionInfo);
        this.logger.debug(`Grace period transaction info: ${JSON.stringify(transactionInfo)}`);
        await this.paymentModel.findOneAndUpdate({ purchaseId: transactionInfo.originalTransactionId }, {
            $set: {
                status: true,
            },
        }, { new: true });
        this.logger.log('Database updated for grace period');
    }
    async handleAutoRenewDisabled(decodedPayload) {
        this.logger.log('Handling auto-renew disabled event');
        const transactionInfo = jwt.decode(decodedPayload.data.signedTransactionInfo);
        this.logger.debug(`Auto-renew disabled transaction info: ${JSON.stringify(transactionInfo)}`);
        await this.paymentModel.findOneAndUpdate({ purchaseId: transactionInfo.originalTransactionId }, {
            $set: {
                status: false,
            },
        }, { new: true });
        this.logger.log('Database updated for auto-renew disabled');
    }
    getDurationFromProductId(productId) {
        switch (productId) {
            case 'com.yourapp.subscription.1month':
                return '1 Month';
            case 'com.yourapp.subscription.3month':
                return '3 Month';
            case 'com.yourapp.subscription.6month':
                return '6 Month';
            case 'com.yourapp.subscription.12month':
                return '12 Month';
            default:
                return '1 Month';
        }
    }
};
exports.PaymentsController = PaymentsController;
__decorate([
    (0, common_1.Post)('handleWebhook'),
    __param(0, (0, common_1.Body)()),
    __param(1, (0, common_1.Res)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], PaymentsController.prototype, "handleWebhook", null);
exports.PaymentsController = PaymentsController = PaymentsController_1 = __decorate([
    (0, common_1.Controller)('payments'),
    __param(0, (0, mongoose_1.InjectModel)(payment_schema_1.Payment.name)),
    __metadata("design:paramtypes", [mongoose_2.Model])
], PaymentsController);
//# sourceMappingURL=payments.controller.js.map